# Generated by Django 5.2.1 on 2025-05-15 18:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("testapp", "0003_anothertestmodel_modelwithbadfastcountquerysets_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="ModelWithDeepManager",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("another_field", models.IntegerField(default=0)),
            ],
        ),
        migrations.CreateModel(
            name="ModelWithIntermediateManager",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field", models.CharField(max_length=50)),
            ],
        ),
    ]
