# Generated by Django 5.2 on 2025-05-14 20:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("testapp", "0002_alter_testmodel_flag"),
    ]

    operations = [
        migrations.CreateModel(
            name="AnotherTestModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name="ModelWithBadFastCountQuerysets",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="ModelWithDynamicallyAssignedManager",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("some_field", models.BooleanField(default=True)),
            ],
        ),
        migrations.CreateModel(
            name="ModelWithSimpleManager",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("data", models.CharField(max_length=10)),
            ],
        ),
    ]
